<?php
interface S2Dao_DtoMetaData {
    const COLUMN_SUFFIX = '_COLUMN';
    public function getBeanClass();
    public function getPropertyTypeSize();
    public function getPropertyType($prop);
    public function hasPropertyType($propertyName);
}

interface S2Dao_CommandContext {
    public function getArg($name);
    public function getArgType($name);
    public function addArg($name, $arg, $argType);
    public function getSql();
    public function getBindVariables();
    public function getBindVariableTypes();
    public function addSql($sql, $bindVariable = null, $bindVariableType = null);
    public function isEnabled();
    public function setEnabled($enabled);
}

interface S2Dao_DaoMetaData {
    const BEAN = 'BEAN';
    const QUERY = 'QUERY';
    const FILE = 'FILE';
    public function getBeanClass();
    public function getBeanMetaData();
    public function hasSqlCommand($methodName);
    public function getSqlCommand($methodName);
    public function createFindCommand($query);
    public function createFindArrayCommand($query);
    public function createFindBeanCommand($query);
    public function createFindYamlCommand($query);
    public function createFindJsonCommand($query);
    public function createFindObjectCommand($query);
}

interface S2Dao_DaoMetaDataFactory {
    public function getDaoMetaData(ReflectionClass $daoClass);
}

interface S2Dao_DaoAnnotationReader {
    const RETURN_OBJ = 0;
    const RETURN_LIST = 1;
    const RETURN_ARRAY = 2;
    const RETURN_YAML = 3;
    const RETURN_JSON = 4;
    const RETURN_MAP = 5;
    public function getBeanClass();
    public function getQuery(ReflectionMethod $method);
    public function getArgNames(ReflectionMethod $method);
    public function getNoPersistentProps(ReflectionMethod $method);
    public function getPersistentProps(ReflectionMethod $method);
    public function getSQL(ReflectionMethod $method, $suffix);
    public function getStoredProcedureName(ReflectionMethod $method);
    public function getReturnType(ReflectionMethod $method);
}

interface S2Dao_Dbms {
    const BIND_TABLE = ':TABLE';
    const BIND_COLUMN = ':COLUMN';
    const BIND_DB = ':DB';
    const BIND_SCHEME = ':SCHEME';
    const BIND_CATALOG = ':CATALOG';
    const BIND_NAME = ':NAME';
    function getAutoSelectSql(S2Dao_BeanMetaData $beanMetaData);
    function getSuffix();
    function getIdentitySelectString();
    function getSequenceNextValString($sequenceName);
    function getTableSql();
    function getTableInfoSql();
    function getPrimaryKeySql();
    function getProcedureNamesSql();
    function getProcedureInfoSql();
    function getLimitOffsetSql();
    function usableLimitOffsetQuery();
}

interface S2Dao_EntityManager {
    public function find($query, $arg1 = null, $arg2 = null, $arg3 = null);
    public function findArray($query, $arg1 = null, $arg2 = null, $arg3 = null);
    public function findBean($query, $arg1 = null, $arg2 = null, $arg3 = null);
    public function findObject($query, $arg1 = null, $arg2 = null, $arg3 = null);
    public function findYaml($query, $arg1 = null, $arg2 = null, $arg3 = null);
    public function findJson($query, $arg1 = null, $arg2 = null, $arg3 = null);
}

interface S2Dao_Node {
    public function getChildSize();
    public function getChild($index);
    public function addChild($node);
    public function accept(S2Dao_CommandContext $ctx);
}

interface S2Dao_BeanMetaData extends S2Dao_DtoMetaData {
    const TABLE = 'TABLE';
    const RELNO_SUFFIX = '_RELNO';
    const RELKEYS_SUFFIX = '_RELKEYS';
    const ID_SUFFIX = '_ID';
    const NO_PERSISTENT_PROPS = 'NO_PERSISTENT_PROPS';
    const VERSION_NO_PROPERTY = 'VERSION_NO_PROPERTY';
    const TIMESTAMP_PROPERTY = 'TIMESTAMP_PROPERTY';
    public function getTableName();
    public function getVersionNoPropertyType();
    public function getVersionNoPropertyName();
    public function hasVersionNoPropertyType();
    public function getTimestampPropertyType();
    public function getTimestampPropertyName();
    public function hasTimestampPropertyType();
    public function convertFullColumnName($alias);
    public function getPropertyTypeByAliasName($aliasName);
    public function getPropertyTypeByColumnName($columnName);
    public function hasPropertyTypeByColumnName($columnName);
    public function hasPropertyTypeByAliasName($aliasName);
    public function getRelationPropertyTypeSize();
    public function getRelationPropertyType($index);
    public function getPrimaryKeySize();
    public function getPrimaryKey($index);
    public function getIdentifierGenerator();
    public function getAutoSelectList();
    public function isRelation();
}

interface S2Dao_SqlCommand {
    public function execute($args);
}

interface S2Dao_IdentifierGenerator {
    public function isSelfGenerate();
}

interface S2Dao_SqlTokenizer {
    const SQL = 1;
    const COMMENT = 2;
    const ELSE_ = 3;
    const BIND_VARIABLE = 4;
    const EOF = 99;
    public function getToken();
    public function getBefore();
    public function getAfter();
    public function getPosition();
    public function getTokenType();
    public function getNextTokenType();
    public function next();
    public function skipToken();
    public function skipWhitespace();
}

interface S2Dao_ProcedureMetaData {
    const STORED_PROCEDURE = 'PROCEDURE';
    const STORED_FUNCTION = 'FUNCTION';
    const INTYPE = 0;
    const OUTTYPE = 1;
    const INOUTTYPE = 9;
    const RETURNTYPE = -1;
    public function getProcedures($catalog = null, $scheme = null, $procedureName);
    public function getProcedureColumnsIn(S2Dao_ProcedureInfo $procedureInfo);
    public function getProcedureColumnsOut(S2Dao_ProcedureInfo $procedureInfo);
    public function getProcedureColumnsInOut(S2Dao_ProcedureInfo $procedureInfo);
    public function getProcedureColumnReturn(S2Dao_ProcedureInfo $procedureInfo);
}
interface S2Dao_ResultSetHandler {
    public function handle(PDOStatement $resultSet);
}

interface S2Dao_StatementFactory {
    public function createStatement(PDO $conn, $sql);
    public function createPreparedStatement(PDO $conn, $sql);
    public function createCallableStatement(PDO $conn, $sql);
}

interface S2Dao_ResultSetFactory {
    public function createResultSet(PDOStatement $ps);
}

interface S2Dao_SelectHandler {
    public function execute($element, $args);
}

interface S2Dao_UpdateHandler {
    public function execute($args, $argTypes = null);
}

interface S2Dao_PropertyType {
    public function getPropertyDesc();
    public function getValueType();
    public function getPropertyName();
    public function getColumnName();
    public function setColumnName($columnName);
    public function isPrimaryKey();
    public function setPrimaryKey($primaryKey);
    public function isPersistent();
    public function setPersistent($persistent);
}

class S2Dao_PropertyTypeImpl implements S2Dao_PropertyType {
    private $propertyDesc = null;
    private $propertyName = null;
    private $columnName = null;
    private $valueType = null;
    private $primaryKey = false;
    private $persistent = true;
    public function __construct($propertyDesc, $valueType = null, $columnName = null) {
        if($propertyDesc instanceof S2Container_PropertyDesc){
            $this->propertyDesc = $propertyDesc;
            $this->propertyName = $propertyDesc->getPropertyName();
            $this->columnName = $propertyDesc->getPropertyName();
            $this->valueType = gettype($propertyDesc->getPropertyType());
        } else if(is_string($propertyDesc)){
            $this->propertyName = $propertyDesc;
            $this->columnName = $propertyDesc;
        }
        if($valueType !== null){
            $this->valueType = $valueType;
        }
        if($columnName !== null){
            $this->columnName = $columnName;
        }
    }
    public function getPropertyDesc() {
        return $this->propertyDesc;
    }
    public function getPropertyName() {
        return $this->propertyName;
    }
    public function getColumnName() {
        return $this->columnName;
    }
    public function setColumnName($columnName) {
        $this->columnName = $columnName;
    }
    public function getValueType() {
        return $this->valueType;
    }
    public function setValueType($type) {
        $this->valueType = $type;
    }
    public function isPrimaryKey() {
        return $this->primaryKey;
    }
    public function setPrimaryKey($primaryKey) {
        $this->primaryKey = $primaryKey;
    }
    public function isPersistent() {
        return $this->persistent;
    }
    public function setPersistent($persistent) {
        $this->persistent = $persistent;
    }
}

interface S2Dao_PHPType {
    const Boolean = 'boolean';
    const Integer = 'integer';
    const Double = 'double';
    const Float = 'float';
    const String = 'string';
    const Object = 'object';
    const Resource = 'resource';
    const Null = 'NULL';
    const Unkwown = 'unknown type';
}

final class S2Dao_PDOType {
    const Boolean = PDO::PARAM_BOOL;
    const Integer = PDO::PARAM_INT;
    const Double = PDO::PARAM_INT;
    const String = PDO::PARAM_STR;
    const Object = PDO::PARAM_STMT;
    const Resource = PDO::PARAM_LOB;
    const Null = PDO::PARAM_NULL;
    const Unkwown = PDO::PARAM_STMT;
    public static function gettype($phpType = null){
        if($phpType === null){
            return self::Null;
        }
        switch($phpType){
            case S2Dao_PHPType::String:
                return self::String;
            case S2Dao_PHPType::Integer:
            case S2Dao_PHPType::Double;
                return self::Integer;
            case S2Dao_PHPType::Boolean:
                return self::Boolean;
            case S2Dao_PHPType::Null:
                return self::Null;
            case S2Dao_PHPType::Resource:
                return self::Resource;
            default:
            case S2Dao_PHPType::Object:
                return self::Object;
        }
    }
}

class S2Dao_BasicHandler {
    private $dataSource_;
    private $sql_;
    private $statementFactory_ = null;
    public function __construct(S2Container_DataSource $ds,
                                $sql,
                                S2Dao_StatementFactory $statementFactory = null) {
        $this->setDataSource($ds);
        $this->setSql($sql);
        if($statementFactory == null){
            $this->setStatementFactory(new S2Dao_BasicStatementFactory());
        } else {
            $this->setStatementFactory($statementFactory);
        }
    }
    public function getDataSource() {
        return $this->dataSource_;
    }
    public function setDataSource(S2Container_DataSource $dataSource) {
        $this->dataSource_ = $dataSource;
    }
    public function getSql() {
        return $this->sql_;
    }
    public function setSql($sql) {
        $this->sql_ = $sql;
    }
    public function getStatementFactory() {
        return $this->statementFactory_;
    }
    public function setStatementFactory($statementFactory) {
        $this->statementFactory_ = $statementFactory;
    }
    private function setAttributes(PDO $connection){
        if(!version_compare(phpversion(), '5.1.3', '>=')){
            return;
        }
        $refPdo = new ReflectionExtension('PDO');
        if(!version_compare($refPdo->getVersion(), '1.0.3', '>=')){
            return;
        }
        if(S2Dao_DbmsManager::getDbms($connection) instanceof S2Dao_MySQL){
            $connection->setAttribute(PDO::ATTR_EMULATE_PREPARES, true);
            $connection->setAttribute(PDO::MYSQL_ATTR_USE_BUFFERED_QUERY, true);
        }
    }
    protected function getConnection() {
        if ($this->dataSource_ == null) {
            throw new S2Container_EmptyRuntimeException('dataSource');
        }
        $conn = $this->dataSource_->getConnection();
        $this->setAttributes($conn);
        return $conn;
    }
    protected function prepareStatement(PDO $connection) {
        if ($this->sql_ == null) {
            throw new S2Container_EmptyRuntimeException('sql');
        }
        return $this->statementFactory_->createPreparedStatement($connection, $this->sql_);
    }
    protected function bindArgs(PDOStatement $ps,
                                array $args = null,
                                array $argTypes = null) {
        if ($args === null) {
            return;
        }
        $c = count($args);
        for ($i = 0; $i < $c; $i++) {
            try {
                $arg = $args[$i];
                $argType = $argTypes[$i];
                $phpType = gettype($arg);
                if($argType !== null){
                    if($this->isNullType($arg, $argType)){
                        $ps->bindValue($i + 1, $arg, S2Dao_PDOType::Null);
                    } else {
                        $ps->bindValue($i + 1, $arg, $this->getBindParamTypes($argType));
                    }
                } else {
                    $ps->bindValue($i + 1, $arg, $this->getBindParamTypes($phpType));
                }
            } catch (Exception $ex) {
                throw new S2Container_SQLRuntimeException($ex);
            }
        }
    }
    protected function getArgTypes($args) {
        if ($args == null) {
            return null;
        }
        $argTypes = array();
        $c = count($args);
        for ($i = 0; $i < $c; ++$i) {
            $arg = $args[$i];
            if ($arg != null) {
                $argTypes[$i] = get_class($arg);
            }
        }
        return $argTypes;
    }
    protected function getCompleteSql($args = null) {
        if ($args == null || !is_array($args)) {
            return $this->sql_;
        }
        $pos = 0;
        $buf = $this->sql_;
        foreach($args as $value){
            $pos = strpos($buf, '?');
            if($pos !== false){
                $buf = substr_replace($buf, $this->getBindVariableText($value), $pos, 1);
            } else {
                break;
            }
        }
        return $buf;
    }
    protected function getBindParamTypes($phpType = null){
        return S2Dao_PDOType::gettype($phpType);
    }
    protected function getBindVariableText($bindVariable) {
        if (is_string($bindVariable)) {
            return "'" . $bindVariable . "'";
        } else if (is_numeric($bindVariable)) {
            return (string)$bindVariable;
        } else if (is_long($bindVariable)) {
            return "'" . date("Y-m-d H.i.s", $bindVariable) . "'";
        } else if (is_bool($bindVariable)) {
            return (string)$bindVariable;
        } else if ($bindVariable == null) {
            return "null";
        } else if (strtotime($bindVariable) !== -1) {
            return "'" . date("Y-m-d", strtotime($bindVariable)) . "'";
        } else {
            return "'" . (string)$bindVariable . "'";
        }
    }
    private function isNullType($value = null, $type = null){
        $nullString = gettype(null);
        if($type == $nullString && gettype($value) == $nullString){
            return true;
        }
        if(!$this->isPrimitive($type)){
            return true;
        }
        return false;
    }
    private function isPrimitive($type){
        return $type != gettype(new stdClass);
    }
}

class S2Dao_BasicResultSetFactory implements S2Dao_ResultSetFactory {
    public function createResultSet(PDOStatement $ps) {
        return new S2Dao_ObjectResultSetHandler();
    }
}

class S2Dao_BasicStatementFactory implements S2Dao_StatementFactory {
    public function createStatement(PDO $conn, $sql){
        return $conn->query($sql);
    }
    public function createPreparedStatement(PDO $conn, $sql) {
        return $conn->prepare($sql);
    }
    public function createCallableStatement(PDO $conn, $sql) {
        return $conn->prepare($sql);
    }
}

class S2Dao_BasicSelectHandler extends S2Dao_BasicHandler implements S2Dao_SelectHandler {
    private static $logger_ = null;
    private $resultSetFactory_ = null;
    private $resultSetHandler_;
    private $fetchSize_ = 100;
    private $maxRows_ = -1;
    public function __construct(S2Container_DataSource $dataSource,
                              $sql,
                              S2Dao_ResultSetHandler $resultSetHandler,
                              S2Dao_StatementFactory $statementFactory = null,
                              S2Dao_ResultSetFactory $resultSetFactory = null) {
        self::$logger_ = S2Container_S2Logger::getLogger(get_class($this));
        $this->setDataSource($dataSource);
        $this->setSql($sql);
        $this->setResultSetHandler($resultSetHandler);
        if($statementFactory == null){
            $this->setStatementFactory(new S2Dao_BasicStatementFactory());
        } else {
            $this->setStatementFactory($statementFactory);
        }
        if($resultSetFactory == null){
            $this->setResultSetFactory(new S2Dao_BasicResultSetFactory());
        } else {
            $this->setResultSetFactory($resultSetFactory);
        }
    }
    public function getResultSetFactory() {
        return $this->resultSetFactory_;
    }
    public function setResultSetFactory(S2Dao_ResultSetFactory $resultSetFactory = null) {
        $this->resultSetFactory_ = $resultSetFactory;
    }
    public function getResultSetHandler() {
        return $this->resultSetHandler_;
    }
    public function setResultSetHandler(S2Dao_ResultSetHandler $resultSetHandler = null) {
        $this->resultSetHandler_ = $resultSetHandler;
    }
    public function getFetchSize() {
        return $this->fetchSize_;
    }
    public function setFetchSize($fetchSize) {
        $this->fetchSize_ = $fetchSize;
    }
    public function getMaxRows() {
        return $this->maxRows_;
    }
    public function setMaxRows($maxRows) {
        $this->maxRows_ = $maxRows;
    }
    public function execute($args1, $args2){
        $stmt = $this->prepareStatement($this->getConnection());
        $stmt->setFetchMode(PDO::FETCH_ASSOC);
        $this->bindArgs($stmt, $args1, $args2);
        if(S2CONTAINER_PHP5_LOG_LEVEL == 1){
            self::$logger_->debug($this->getCompleteSql($args1));
        }
        if ($this->resultSetHandler_ == null) {
            throw new S2Container_EmptyRuntimeException('resultSetHandler');
        }
        try{
            $stmt->execute();
            $resultSet = $this->createResultSet($stmt);
            $columnCount = $stmt->columnCount();
            if($columnCount == 1){
                $rs = $stmt->fetch(PDO::FETCH_NUM);
                return (int)$rs[0];
            } else if($columnCount === null || $columnCount <= 0){
                return null;
            } else {
                return $this->resultSetHandler_->handle($stmt);
            }
        } catch (PDOException $ex) {
            throw new S2Dao_SQLRuntimeException($ex);
        }
    }
    protected function setup($con, $args) {
        return $args;
    }
    protected function setupDatabaseMetaData(DatabaseMetaData $dbMetaData) {
    }
    protected function createResultSet(PDOStatement $stmt) {
        return $this->resultSetFactory_->createResultSet($stmt);
    }
}

class S2Dao_BasicUpdateHandler
    extends S2Dao_BasicHandler 
    implements S2Dao_UpdateHandler {
    private static $logger_ = null;
    public function __construct(S2Container_DataSource $dataSource,
                                $sql,
                                S2Dao_StatementFactory $statementFactory = null) {
        parent::__construct($dataSource, $sql, $statementFactory);
        self::$logger_ = S2Container_S2Logger::getLogger(get_class($this));
    }
    public function execute($args, $argsTypes = null){
        $stmt = $this->prepareStatement($this->getConnection());
        $this->bindArgs($stmt, $args, $argsTypes);
        if(S2CONTAINER_PHP5_LOG_LEVEL == 1){
            self::$logger_->debug($this->getCompleteSql($args));
        }
        try {
            $stmt->execute();
            return $stmt->rowCount();
        } catch (PDOException $e) {
            throw new S2Dao_SQLRuntimeException($e);
        }
    }
}

abstract class S2Dao_AbstractSqlCommand implements S2Dao_SqlCommand {
    private $dataSource_;
    private $statementFactory_;
    private $sql_;
    private $notSingleRowUpdatedExceptionClass_;
    public function __construct(S2Container_DataSource $dataSource,
                                S2Dao_StatementFactory $statementFactory = null) {
        $this->dataSource_ = $dataSource;
        $this->statementFactory_ = $statementFactory;
    }
    public function getDataSource() {
        return $this->dataSource_;
    }
    public function getStatementFactory() {
        return $this->statementFactory_;
    }
    public function getSql() {
        return $this->sql_;
    }
    public function setSql($sql) {
        $this->sql_ = $sql;
    }
    public function getNotSingleRowUpdatedExceptionClass() {
        return $this->notSingleRowUpdatedExceptionClass_;
    }
    public function setNotSingleRowUpdatedExceptionClass($notSingleRowUpdatedExceptionClass) {
        $this->notSingleRowUpdatedExceptionClass_ = $notSingleRowUpdatedExceptionClass;
    }
}

abstract class S2Dao_AbstractStaticCommand extends S2Dao_AbstractSqlCommand {
    private $beanMetaData_;
    public function __construct(S2Container_DataSource $dataSource,
                                S2Dao_StatementFactory $statementFactory = null,
                                S2Dao_BeanMetaData $beanMetaData) {
        parent::__construct($dataSource, $statementFactory);
        $this->beanMetaData_ = $beanMetaData;
    }
    public function getBeanMetaData() {
        return $this->beanMetaData_;
    }
}

abstract class S2Dao_AbstractAutoHandler extends S2Dao_BasicHandler implements S2Dao_UpdateHandler {
    private static $logger_ = null;
    private $beanMetaData_;
    private $bindVariables_;
    private $bindVariableTypes_;
    private $timestamp_;
    private $versionNo_;
    private $propertyTypes_;
    private $beanCache_ = null;
    public function __construct(S2Container_DataSource $dataSource,
                                S2Dao_StatementFactory $statementFactory,
                                S2Dao_BeanMetaData $beanMetaData,
                                $propertyTypes) {
        self::$logger_ = S2Container_S2Logger::getLogger(get_class($this));
        $this->setDataSource($dataSource);
        $this->setStatementFactory($statementFactory);
        $this->beanMetaData_ = $beanMetaData;
        $this->propertyTypes_ = $propertyTypes;
    }
    public function getBeanMetaData() {
        return $this->beanMetaData_;
    }
    protected static function getLogger() {
        return self::$logger_;
    }
    protected function getBindVariables() {
        return $this->bindVariables_;
    }
    protected function setBindVariables($bindVariables) {
        $this->bindVariables_ = $bindVariables;
    }
    protected function getBindVariableTypes() {
        return $this->bindVariableTypes_;
    }
    protected function setBindVariableTypes($types) {
        $this->bindVariableTypes_ = $types;
    }
    protected function getTimestamp() {
        return $this->timestamp_;
    }
    protected function setTimestamp($timestamp) {
        $this->timestamp_ = $timestamp;
    }
    protected function getVersionNo() {
        return $this->versionNo_;
    }
    protected function setVersionNo($versionNo) {
        $this->versionNo_ = $versionNo;
    }
    protected function getPropertyTypes() {
        return $this->propertyTypes_;
    }
    protected function setPropertyTypes($propertyTypes) {
        $this->propertyTypes_ = $propertyTypes;
    }
    public function execute($args, $arg2 = null) {
        if(is_array($args)){
            $bean = $args[0];
            $this->beanCache_ = $bean;
            $this->preUpdatePropertyTypes($bean);
            $this->preUpdateBean($bean);
            $this->setupBindVariables($bean);
            $this->setupBindValiablesType($bean);
            if(S2CONTAINER_PHP5_LOG_LEVEL == 1){
                $this->getLogger()->debug(
                    $this->getCompleteSql($this->bindVariables_)
                );
            }
            return $this->execute($this->getConnection(), false);
        } else if($args instanceof PDO && is_bool($arg2)){
            $ps = $this->prepareStatement($args);
            $ps->setFetchMode(PDO::FETCH_ASSOC);
            return $this->execute($ps, $arg2);
        } else if($args instanceof PDOStatement && is_bool($arg2)){
            try {
                $this->bindArgs($args, $this->bindVariables_, $this->bindVariableTypes_);
                $result = $args->execute();
            } catch(Exception $e){
                throw $e;
            }
            $ret = -1;
            if($result === false){
                $this->getLogger()->error($args->errorCode(), __METHOD__);
                $this->getLogger()->error(print_r($args->errorInfo()), __METHOD__);
                unset($args);
                throw new Exception();
            } else {
                $ret = $args->rowCount();
            }
            $this->postUpdateBean($this->beanCache_);
            $this->postUpdatePropertyTypes($this->beanCache_);
            return $ret;
        }
    }
    protected function preUpdateBean($bean) {
    }
    protected function postUpdateBean($bean) {
    }
    protected abstract function setupBindVariables($bean);
    protected function preUpdatePropertyTypes($bean){
        $bmd = $this->getBeanMetaData();
        $c = $bmd->getPropertyTypeSize();
        for($i = 0; $i < $c; $i++){
            $pt = $bmd->getPropertyType($i);
            $pd = $pt->getPropertyDesc();
            $value = $pd->getValue($bean);
            $pt->setValueType(gettype($value));
        }
    }
    protected function setupBindValiablesType($bean){
        $c = count($this->bindVariableTypes_);
        for($i = 0; $i < $c; $i++){
            $type =& $this->bindVariableTypes_[$i];
            if($type == gettype(null)){
                $type = gettype($this->bindVariables_[$i]);
            }
        }
    }
    protected function postUpdatePropertyTypes($bean){
        $this->preUpdatePropertyTypes($bean);
    }
    protected function setupInsertBindVariables($bean) {
        $varList = new S2Dao_ArrayList();
        $varTypeList = new S2Dao_ArrayList();
        $bmd = $this->getBeanMetaData();
        $timestampPropertyName = $bmd->getTimestampPropertyName();
        $versionNoPropertyName = $bmd->getVersionNoPropertyName();
        $c = count($this->propertyTypes_);
        for ($i = 0; $i < $c; ++$i) {
            $pt = $this->propertyTypes_[$i];
            $propName = $pt->getPropertyName();
            if (strcasecmp($propName, $timestampPropertyName) === 0) {
                $this->setTimestamp(date('Y-m-d H:i:s', time()));
                $varList->add($this->getTimestamp());
            } else if (strcmp($propName, $versionNoPropertyName) === 0) {
                $this->setVersionNo(0);
                $varList->add($this->getVersionNo());
            } else {
                $varList->add($pt->getPropertyDesc()->getValue($bean));
            }
            $varTypeList->add($pt->getValueType());
        }
        $this->setBindVariables($varList->toArray());
        $this->setBindVariableTypes($varTypeList->toArray());
    }
    protected function setupUpdateBindVariables($bean) {
        $varList = new S2Dao_ArrayList();
        $varTypeList = new S2Dao_ArrayList();
        $bmd = $this->getBeanMetaData();
        $timestampPropertyName = $bmd->getTimestampPropertyName();
        $versionNoPropertyName = $bmd->getVersionNoPropertyName();
        $c = count($this->propertyTypes_);
        for ($i = 0; $i < $c; ++$i) {
            $pt = $this->propertyTypes_[$i];
            $propName = $pt->getPropertyName();
            if (strcasecmp($propName, $timestampPropertyName) == 0) {
                $this->setTimestamp(date('Y-m-d H:i:s', time()));
                $varList->add($this->getTimestamp());
            } else if (strcmp($propName, $versionNoPropertyName) == 0) {
                $value = $pt->getPropertyDesc()->getValue($bean);
                $intValue = (int)$value + 1;
                $this->setVersionNo($intValue);
                $varList->add($this->getVersionNo());
            } else {
                $varList->add($pt->getPropertyDesc()->getValue($bean));
            }
            $varTypeList->add($pt->getValueType());
        }
        $this->addAutoUpdateWhereBindVariables($varList, $varTypeList, $bean);
        $this->setBindVariables($varList->toArray());
        $this->setBindVariableTypes($varTypeList->toArray());
    }
    protected function setupDeleteBindVariables($bean) {
        $varList = new S2Dao_ArrayList();
        $varTypeList = new S2Dao_ArrayList();
        $this->addAutoUpdateWhereBindVariables($varList, $varTypeList, $bean);
        $this->setBindVariables($varList->toArray());
        $this->setBindVariableTypes($varTypeList->toArray());
    }
    protected function addAutoUpdateWhereBindVariables(S2Dao_ArrayList $varList,
                                                       S2Dao_ArrayList $varTypeList,
                                                       $bean) {
        $bmd = $this->getBeanMetaData();
        for ($i = 0; $i < $bmd->getPrimaryKeySize(); ++$i) {
            $pt = $bmd->getPropertyTypeByColumnName($bmd->getPrimaryKey($i));
            $varList->add($pt->getPropertyDesc()->getValue($bean));
            $varTypeList->add($pt->getValueType());
        }
        if ($bmd->hasVersionNoPropertyType()) {
            $pt = $bmd->getVersionNoPropertyType();
            $varList->add($pt->getPropertyDesc()->getValue($bean));
            $varTypeList->add($pt->getValueType());
        }
        if ($bmd->hasTimestampPropertyType()) {
            $pt = $bmd->getTimestampPropertyType();
            $varList->add($pt->getPropertyDesc()->getValue($bean));
            $varTypeList->add($pt->getValueType());
        }
    }
    protected function updateTimestampIfNeed($bean) {
        if ($this->getTimestamp() !== null) {
            $pd = $this->getBeanMetaData()->getTimestampPropertyType()->getPropertyDesc();
            $pd->setValue($bean, $this->getTimestamp());
        }
    }
    protected function updateVersionNoIfNeed($bean) {
        if ($this->getVersionNo() !== null) {
            $pd = $this->getBeanMetaData()->getVersionNoPropertyType()->getPropertyDesc();
            $pd->setValue($bean, (int)$this->getVersionNo());
        }
    }
}

abstract class S2Dao_AbstractAutoStaticCommand extends S2Dao_AbstractStaticCommand {
    private $propertyTypes_ = array();
    public function __construct(S2Container_DataSource $dataSource,
                                S2Dao_StatementFactory $statementFactory = null,
                                S2Dao_BeanMetaData $beanMetaData,
                                $propertyNames) {
        parent::__construct($dataSource, $statementFactory, $beanMetaData);
        $this->setupPropertyTypes($propertyNames);
        $this->setupSql();
    }
    public function execute($args) {
        $handler = $this->createAutoHandler();
        $handler->setSql($this->getSql());
        $rows = $handler->execute($args);
        if ($rows != 1) {
            throw new S2Dao_NotSingleRowUpdatedRuntimeException(get_class($args[0]), $rows);
        }
        return (int)$rows;
    }
    protected function getPropertyTypes() {
        return $this->propertyTypes_;
    }
    protected function setPropertyTypes($propertyTypes) {
        $this->propertyTypes_ = $propertyTypes;
    }
    protected abstract function createAutoHandler();
    protected abstract function setupPropertyTypes($propertyNames);
    protected function setupInsertPropertyTypes(array $propertyNames) {
        $types = new S2Dao_ArrayList();
        $c = count($propertyNames);
        for ($i = 0; $i < $c; ++$i) {
            $pt = $this->getBeanMetaData()->getPropertyType($propertyNames[$i]);
            if ($pt->isPrimaryKey() &&
                !$this->getBeanMetaData()->getIdentifierGenerator()->isSelfGenerate()) {
                continue;
            }
            $types->add($pt);
        }
        $this->propertyTypes_ = $types->toArray();
    }
    protected function setupUpdatePropertyTypes($propertyNames) {
        $types = new S2Dao_ArrayList();
        $c = count($propertyNames);
        for ($i = 0; $i < $c; ++$i) {
            $pt = $this->getBeanMetaData()->getPropertyType($propertyNames[$i]);
            if ($pt->isPrimaryKey()) {
                continue;
            }
            $types->add($pt);
        }
        $this->propertyTypes_ = $types->toArray();
    }
    protected function setupDeletePropertyTypes($propertyNames){}
    protected abstract function setupSql();
    protected function setupInsertSql() {
        $buf = '';
        $buf .= 'INSERT INTO ';
        $buf .= $this->getBeanMetaData()->getTableName();
        $buf .= ' (';
        $c = count($this->propertyTypes_);
        for ($i = 0; $i < $c; ++$i) {
            $pt = $this->propertyTypes_[$i];
            $buf .= $pt->getColumnName();
            $buf .= ', ';
        }
        $buf = preg_replace('/(, )$/', '', $buf);
        $buf .= ') VALUES (';
        for ($i = 0; $i < $c; ++$i) {
            $buf .= '?, ';
        }
        $buf = preg_replace('/(, )$/', '', $buf);
        $buf .= ')';
        $this->setSql($buf);
    }
    protected function setupUpdateSql() {
        $this->checkPrimaryKey();
        $buf = '';
        $buf .= 'UPDATE ';
        $buf .= $this->getBeanMetaData()->getTableName();
        $buf .= ' SET ';
        $c = count($this->propertyTypes_);
        for ($i = 0; $i < $c; ++$i) {
            $pt = $this->propertyTypes_[$i];
            $buf .= $pt->getColumnName();
            $buf .= ' = ?, ';
        }
        $buf = preg_replace('/(, )$/', '', $buf);
        $this->setupUpdateWhere($buf);
        $this->setSql($buf);
    }
    protected function setupDeleteSql() {
        $this->checkPrimaryKey();
        $buf = '';
        $buf .= 'DELETE FROM ';
        $buf .= $this->getBeanMetaData()->getTableName();
        $this->setupUpdateWhere($buf);
        $this->setSql($buf);
    }
    protected function checkPrimaryKey() {
        $bmd = $this->getBeanMetaData();
        if ($bmd->getPrimaryKeySize() == 0) {
            throw new S2Dao_PrimaryKeyNotFoundRuntimeException($bmd->getBeanClass());
        }
    }
    protected function setupUpdateWhere(&$buf) {
        $bmd = $this->getBeanMetaData();
        $buf .= ' WHERE ';
        for ($i = 0; $i < $bmd->getPrimaryKeySize(); ++$i) {
            $buf .= $bmd->getPrimaryKey($i);
            $buf .= ' = ? AND ';
        }
        $buf = preg_replace('/( AND )$/', '', $buf);
        if ($bmd->hasVersionNoPropertyType()) {
            $pt = $bmd->getVersionNoPropertyType();
            $buf .= ' AND ';
            $buf .= $pt->getColumnName();
            $buf .= ' = ?';
        }
        if ($bmd->hasTimestampPropertyType()) {
            $pt = $bmd->getTimestampPropertyType();
            $buf .= ' AND ';
            $buf .= $pt->getColumnName();
            $buf .= ' = ?';
        }
    }
}

abstract class S2Dao_AbstractBeanMetaDataResultSetHandler implements S2Dao_ResultSetHandler {
    private $beanMetaData_;
    public function __construct(S2Dao_BeanMetaData $beanMetaData) {
        $this->beanMetaData_ = $beanMetaData;
    }
    public function getBeanMetaData() {
        return $this->beanMetaData_;
    }
    protected function createRow(array $resultSet){
        $row = $this->beanMetaData_->getBeanClass()->newInstance();
        $columnNames = new S2Dao_ArrayList(array_keys($resultSet));
        $c = $this->beanMetaData_->getPropertyTypeSize();
        for($i = 0; $i < $c; ++$i) {
            $pt = $this->beanMetaData_->getPropertyType($i);
            if ($columnNames->contains($pt->getColumnName())) {
                $value = $resultSet[$pt->getColumnName()];
                $pd = $pt->getPropertyDesc();
                $pd->setValue($row, $value);
            } else if ($columnNames->contains($pt->getPropertyName())) {
                $value = $resultSet[$pt->getPropertyName()];
                $pd = $pt->getPropertyDesc();
                $pd->setValue($row, $value);
            } else if (!$pt->isPersistent()) {
                $iter = $columnNames->iterator();
                for (; $iter->valid(); $iter->next()) {
                    $columnName = $iter->current();
                    $columnName2 = str_replace('_', '', $columnName);
                    if (strcasecmp($columnName2, $pt->getColumnName()) == 0) {
                        $value = $resultSet[$pt->getColumnName()];
                        $pd = $pt->getPropertyDesc();
                        $pd->setValue($row, $value);
                        break;
                    }
                }
            }
        }
        return $row;
    }
    protected function createRelationRow(S2Dao_RelationPropertyType $rpt,
                                         array $resultSet = null,
                                         S2Dao_HashMap $relKeyValues = null){
        if($resultSet == null && $relKeyValues == null){
            return $rpt->getPropertyDesc()->getPropertyType()->newInstance();
        }
        $row = null;
        $columnNames = new S2Dao_ArrayList(array_keys($resultSet));
        $bmd = $rpt->getBeanMetaData();
        for ($i = 0; $i < $rpt->getKeySize(); ++$i) {
            $columnName = $rpt->getMyKey($i);
            if($this->columnContains($columnNames, $columnName)) {
                if ($row === null) {
                    $row = $this->createRelationRow($rpt);
                }
                if ($relKeyValues != null && $relKeyValues->containsKey($columnName)) {
                    $value = $relKeyValues->get($columnName);
                    $pt = $bmd->getPropertyTypeByColumnName($rpt->getYourKey($i));
                    $pd = $pt->getPropertyDesc();
                    if ($value !== null) {
                        $pd->setValue($row, $value);
                    }
                }
            }
            continue;
        }
        $c = $bmd->getPropertyTypeSize();
        $existColumn = 0;
        for ($i = 0; $i < $c; ++$i) {
            $pt = $bmd->getPropertyType($i);
            $columnName = $pt->getColumnName() . '_' . $rpt->getRelationNo();
            if(!$this->columnContains($columnNames, $columnName)){
                continue;
            }
            $existColumn++;
            if ($row === null) {
                $row = $this->createRelationRow($rpt);
            }
            $value = null;
            if ($relKeyValues !== null && $relKeyValues->containsKey($columnName)) {
                $value = $relKeyValues->get($columnName);
            } else {
                $value = $resultSet[$columnName];
            }
            $pd = $pt->getPropertyDesc();
            if ($value !== null) {
                $pd->setValue($row, $value);
            }
        }
        if($existColumn == 0){
            return null;
        }
        return $row;
    }
    private function columnContains($columnNames, $column){
        if(is_string($columnNames)){
            if(strcasecmp($columnNames, $column) == 0){
                return true;
            }
            return false;
        }
        $c = count($columnNames);
        for($i = 0; $i < $c; $i++){
            if($this->columnContains($columnNames[$i], $column)){
                return true;
            }
        }
        return false;
    }
}

abstract class S2Dao_AbstractDao {
    private $entityManager_;
    public function __construct(S2Dao_DaoMetaDataFactory $daoMetaDataFactory) {
        $this->entityManager_ = new S2Dao_EntityManagerImpl(
                    $daoMetaDataFactory->getDaoMetaData(new ReflectionClass($this))
                    );
    }
    public function getEntityManager() {
        return $this->entityManager_;
    }
}

abstract class S2Dao_AbstractDynamicCommand extends S2Dao_AbstractSqlCommand {
    private $rootNode_;
    private $argNames_ = array();
    private $argTypes_ = array();
    public function __construct(S2Container_DataSource $dataSource,
                                S2Dao_StatementFactory $statementFactory = null){
        parent::__construct($dataSource, $statementFactory);
    }
    public function setSql($sql) {
        parent::setSql($sql);
        $sqlparser = new S2Dao_SqlParserImpl($sql);
        $this->rootNode_ = $sqlparser->parse();
    }
    public function getArgNames() {
        return $this->argNames_;
    }
    public function setArgNames($argNames) {
        $this->argNames_ = $argNames;
    }
    public function getArgTypes() {
        return $this->argTypes_;
    }
    public function setArgTypes($argTypes) {
        $this->argTypes_ = $argTypes;
    }
    protected function apply($args) {
        $ctx = $this->createCommandContext($args);
        $this->rootNode_->accept($ctx);
        return $ctx;
    }
    protected function createCommandContext($args) {
        $ctx = new S2Dao_CommandContextImpl();
        if ($args != null) {
            $c = count($args);
            for ($i = 0; $i < $c; ++$i) {
                $argType = null;
                if ($args[$i] !== null){
                    if ($i < count($this->argTypes_)) {
                        $argType = $this->argTypes_[$i];
                    } else if ($args[$i] !== null) {
                        $argType = gettype($args[$i]);
                    }
                }
                if ($i < count($this->argNames_)) {
                    $ctx->addArg($this->argNames_[$i], $args[$i], $argType);
                } else {
                    $ctx->addArg('$' . ($i + 1), $args[$i], $argType);
                }
            }
        }
        return $ctx;
    }
}

interface S2Dao_AnnotationReaderFactory {
    public function createDaoAnnotationReader(S2Container_BeanDesc $daoBeanDesc);
    public function createBeanAnnotationReader($beanClass);
}

abstract class S2Dao_AbstractAnnotationReader {
    protected $reader = null;
    public function __construct(S2Container_BeanDesc $beandesc) {
        if(defined('S2DAO_PHP5_USE_COMMENT')){
            if(S2DAO_PHP5_USE_COMMENT === true){
                $this->setCommentAnnotationHandler();
                $this->reader = $this->createCommentAnnotationReader($beandesc);
            }
        }
        if($this->reader === null){
            $this->reader = $this->createConstantAnnotationReader($beandesc);
        }
    }
    abstract protected function createConstantAnnotationReader($beandesc);
    abstract protected function createCommentAnnotationReader($beandesc);
    public function __call($name, $param){
        return call_user_func_array(array($this->reader, $name), $param);
    }
    private function setCommentAnnotationHandler(){
        if(class_exists('S2Container_AnnotationContainer', false)){
            if(strcasecmp(S2Container_AnnotationContainer::$DEFAULT_ANNOTATION_READER,
                      'S2DaoAnnotationReader') != 0){
               S2Container_AnnotationContainer::$DEFAULT_ANNOTATION_READER = 'S2DaoAnnotationReader';
            }
        }
    }
}

class S2Dao_FieldAnnotationReaderFactory implements S2Dao_AnnotationReaderFactory {
    public function createDaoAnnotationReader(S2Container_BeanDesc $daoBeanDesc) {
        return new S2Dao_FieldDaoAnnotationReader($daoBeanDesc);
    }
    public function createBeanAnnotationReader($beanClass) {
        return new S2Dao_FieldBeanAnnotationReader($beanClass);
    }
}

final class S2Dao_DbmsManager {
    private static $dbmses = null;
    private static $staticConst = false;
    private function S2Dao_DbmsManager() {
    }
    private static function staticConst(){
        self::$dbmses = new S2Dao_HashMap();
        $dbmsClassNames = parse_ini_file(S2DAO_PHP5 . '/dbms.properties');
        foreach($dbmsClassNames as $key => $value){
            self::$dbmses->put(strtolower($key), new $value);
        }
        self::$staticConst = true;
    }
    public static function getDbms(PDO $ds) {
        if(!self::$staticConst){
            self::staticConst();
        }
        return self::$dbmses->get($ds->getAttribute(PDO::ATTR_DRIVER_NAME));
    }
}

class S2Dao_Standard implements S2Dao_Dbms {
    const baseSqlPattern = '/^.*?(select)/i';
    private $autoSelectFromClauseCache_ = null;
    public function __construct(){
        $this->autoSelectFromClauseCache_ = new S2Dao_HashMap();
    }
    public function getSuffix() {
        return '';
    }
    public function getAutoSelectSql(S2Dao_BeanMetaData $beanMetaData) {
        $buf = '';
        $buf .= $beanMetaData->getAutoSelectList();
        $buf .= ' ';
        $beanName = $beanMetaData->getBeanClass()->getName();
        $fromClause = $this->autoSelectFromClauseCache_->get($beanName);
        if ($fromClause == null) {
            $fromClause = $this->createAutoSelectFromClause($beanMetaData);
            $this->autoSelectFromClauseCache_->put($beanName, $fromClause);
        }
        $buf .= $fromClause;
        return $buf;
    }
    protected function createAutoSelectFromClause(S2Dao_BeanMetaData $beanMetaData) {
        $buf = 'FROM ';
        $myTableName = $beanMetaData->getTableName();
        $buf .= $myTableName;
        for ($i = 0; $i < $beanMetaData->getRelationPropertyTypeSize(); ++$i) {
            $rpt = $beanMetaData->getRelationPropertyType($i);
            $bmd = $rpt->getBeanMetaData();
            $buf .= ' LEFT OUTER JOIN ';
            $buf .= $bmd->getTableName();
            $buf .= ' ';
            $yourAliasName = $rpt->getPropertyName();
            $buf .= $yourAliasName;
            $buf .= ' ON ';
            for ($j = 0; $j < $rpt->getKeySize(); ++$j) {
                $buf .= $myTableName;
                $buf .= '.';
                $buf .= $rpt->getMyKey($j);
                $buf .= ' = ';
                $buf .= $yourAliasName;
                $buf .= '.';
                $buf .= $rpt->getYourKey($j);
                $buf .= ' AND ';
            }
            $buf = preg_replace('/( AND )$/', '', $buf);
        }
        return $buf;
    }
    public function isSelfGenerate() {
        return true;
    }
    public function getBaseSql($sql) {
        if (preg_match(self::baseSqlPattern, $sql, $m)) {
            return $m[1];
        }
        return $sql;
    }
    public function getIdentitySelectString() {
        return null;
    }
    public function getSequenceNextValString($sequenceName) {
        return null;
    }
    public function getTableSql(){
        return null;
    }
    public function getTableInfoSql(){
        return null;
    }
    public function getPrimaryKeySql(){
        return null;
    }
    public function getProcedureNamesSql(){
        return null;
    }
    public function getProcedureInfoSql(){
        return null;
    }
    public function getLimitOffsetSql(){
        return null;
    }
    public function usableLimitOffsetQuery() {
        return false;
    }
}

abstract class S2Dao_AbstractIdentifierGenerator implements S2Dao_IdentifierGenerator {
    private static $resultSetHandler_;
    private $propertyName_;
    private $dbms_;
    public function __construct($propertyName, S2Dao_Dbms $dbms) {
        self::$resultSetHandler_ = new S2Dao_ObjectResultSetHandler();
        $this->propertyName_ = $propertyName;
        $this->dbms_ = $dbms;
    }
    public function getPropertyName() {
        return $this->propertyName_;
    }
    public function getDbms() {
        return $this->dbms_;
    }
    protected function executeSql(S2Container_DataSource $ds, $sql, $args) {
        $handler = new S2Dao_BasicSelectHandler($ds, $sql, self::$resultSetHandler_);
        return $handler->execute($args, null);
    }
    public abstract function setIdentifier($bean, S2Container_DataSource $value);
    public function setIdentifier2($bean, $value) {
        if ($this->propertyName_ == null) {
            throw new S2Container_EmptyRuntimeException('propertyName');
        }
        $class = new ReflectionClass(get_class($bean));
        $beanDesc = S2Container_BeanDescFactory::getBeanDesc($class);
        $pd = $beanDesc->getPropertyDesc($this->propertyName_);
        $pd->setValue($bean, $value);
    }
}

class S2Dao_EntityManagerImpl implements S2Dao_EntityManager {
    private static $EMPTY_ARGS = array();
    private $daoMetaData_;
    public function __construct(S2Dao_DaoMetaData $daoMetaData) {
        $this->daoMetaData_ = $daoMetaData;
    }
    public function getDaoMetaData() {
        return $this->daoMetaData_;
    }
    public function find($query, $args = null, $arg2 = null, $arg3 = null) {
        switch(func_num_args()){
            case 1:
                return $this->find($query, self::$EMPTY_ARGS);
            case 2:
                if(is_array($args)){
                    $cmd = $this->daoMetaData_->createFindCommand($query);
                    return $cmd->execute($args);
                }
                return $this->find($query, array($args));
            case 3:
                return $this->find($query, array($args, $arg2));
            case 4:
                return $this->find($query, array($args, $arg2, $arg3));
        }
    }
    public function findArray($query, $args = null, $arg2 = null, $arg3 = null) {
        switch(func_num_args()){
            case 1:
                return $this->findArray($query, self::$EMPTY_ARGS);
            case 2:
                if(is_array($args)){
                    $cmd = $this->daoMetaData_->createFindArrayCommand($query);
                    return $cmd->execute($args);
                }
                return $this->findArray($query, array($args));
            case 3:
                return $this->findArray($query, array($args, $arg2));
            case 4:
                return $this->findArray($query, array($args, $arg2, $arg3));
        }
    }
    public function findBean($query, $args = null, $arg2 = null, $arg3 = null) {
        switch(func_num_args()){
            case 1:
                return $this->findBean($query, self::$EMPTY_ARGS);
            case 2:
                if(is_array($args)){
                    $cmd = $this->daoMetaData_->createFindBeanCommand($query);
                    return $cmd->execute($args);
                }
                return $this->findBean($query, array($args));
            case 3:
                return $this->findBean($query, array($args, $arg2));
            case 4:
                return $this->findBean($query, array($args, $arg2, $arg3));
        }
    }
    public function findYaml($query, $args = null, $arg2 = null, $arg3 = null) {
       switch(func_num_args()){
            case 1:
                return $this->findYaml($query, self::$EMPTY_ARGS);
            case 2:
                if(is_array($args)){
                    $cmd = $this->daoMetaData_->createFindYamlCommand($query);
                    return $cmd->execute($args);
                }
                return $this->findYaml($query, array($args));
            case 3:
                return $this->findYaml($query, array($args, $arg2));
            case 4:
                return $this->findYaml($query, array($args, $arg2, $arg3));
        }
    }
    public function findJson($query, $args = null, $arg2 = null, $arg3 = null) {
       switch(func_num_args()){
            case 1:
                return $this->findJson($query, self::$EMPTY_ARGS);
            case 2:
                if(is_array($args)){
                    $cmd = $this->daoMetaData_->createFindJsonCommand($query);
                    return $cmd->execute($args);
                }
                return $this->findJson($query, array($args));
            case 3:
                return $this->findJson($query, array($args, $arg2));
            case 4:
                return $this->findJson($query, array($args, $arg2, $arg3));
        }
    }
    public function findObject($query, $args = null, $arg2 = null, $arg3 = null) {
        switch(func_num_args()){
            case 1:
                return $this->findObject($query, self::$EMPTY_ARGS);
            case 2:
                if(is_array($args)){
                    $cmd = $this->daoMetaData_->createFindObjectCommand($query);
                    return $cmd->execute($args);
                } else {
                    return $this->findObject($query, array($args));
                }
            case 3:
                return $this->findObject($query, array($args, $arg2));
            case 4:
                return $this->findObject($query, array($args, $arg2, $arg3));
        }
    }
}

class S2Dao_DaoMetaDataFactoryImpl implements S2Dao_DaoMetaDataFactory {
    protected $daoMetaDataCache;
    protected $dataSource;
    protected $statementFactory;
    protected $resultSetFactory;
    protected $readerFactory;
    public function __construct(S2Container_DataSource $dataSource,
                                S2Dao_StatementFactory $statementFactory = null,
                                S2Dao_ResultSetFactory $resultSetFactory = null,
                                S2Dao_AnnotationReaderFactory $readerFactory) {
        $this->daoMetaDataCache = new S2Dao_HashMap();
        $this->dataSource = $dataSource;
        $this->statementFactory = $this->createStatementFactory($statementFactory);
        $this->resultSetFactory = $this->createResultSetFactory($resultSetFactory);
        $this->readerFactory = $readerFactory;
    }
    public function createStatementFactory(S2Dao_StatementFactory $statementFactory = null){
        if($statementFactory === null){
            return new S2Dao_BasicStatementFactory();
        }
        return $statementFactory;
    }
    public function createResultSetFactory(S2Dao_ResultSetFactory $resultSetFactory = null){
        if($resultSetFactory === null){
            return new S2Dao_BasicResultSetFactory();
        }
        return $resultSetFactory;
    }
    public function getDaoMetaData(ReflectionClass $daoClass) {
        $key = $daoClass->getName();
        $dmd = $this->daoMetaDataCache->get($key);
        if ($dmd !== null) {
            return $dmd;
        }
        $dmd = new S2Dao_DaoMetaDataImpl($daoClass,
                                         $this->dataSource,
                                         $this->statementFactory,
                                         $this->resultSetFactory,
                                         $this->readerFactory);
        $this->daoMetaDataCache->put($key, $dmd);
        return $dmd;
    }
}

class S2DaoAssertAtLeastOneRowInterceptor extends S2Container_AbstractInterceptor {
    public function invoke(S2Container_MethodInvocation $invocation){
        $result = $invocation->proceed();
        if (is_numeric($result)) {
            $rows = (int)$result;
            if ($rows < 1) {
                throw new S2Dao_NoRowsUpdatedRuntimeException();
            }
        }
        return $result;
    }
}

class S2DaoAssertExactlyOneRowInterceptor extends S2Container_AbstractInterceptor {
    public function invoke(S2Container_MethodInvocation $invocation) {
        $result = $invocation->proceed();
        if (is_numeric($result)) {
            $rows = (int)$result;
            if ($rows != 1) {
                throw new S2Dao_NotExactlyOneRowUpdatedRuntimeException($rows);
            }
        }
        return $result;
    }
}

class S2DaoInterceptor extends S2Container_AbstractInterceptor {
    private $daoMetaDataFactory_;
    public function __construct(S2Dao_DaoMetaDataFactory $daoMetaDataFactory) {
        $this->daoMetaDataFactory_ = $daoMetaDataFactory;
    }
    public function invoke(S2Container_MethodInvocation $invocation){
        try{
            $method = $invocation->getMethod();
            if (!S2Container_MethodUtil::isAbstract($method)) {
                return $invocation->proceed();
            }
            $targetClass = $this->getTargetClass($invocation);
            $dmd = $this->daoMetaDataFactory_->getDaoMetaData($targetClass);
            $cmd = $dmd->getSqlCommand($method->getName());
            $ret = $cmd->execute($invocation->getArguments());
            return $ret;
        } catch (Exception $e){
            throw $e;
        }
    }
}

class S2Dao_ArrayList extends ArrayObject {
    public function __construct($object = null){
        if($object === null){
            parent::__construct();
        } else {
            parent::__construct($object);
        }
    }
    public function size(){
        return $this->count();
    }
    public function isEmpty(){
        return count($this) == 0;
    }
    public function contains($object){
        return in_array($object, $this->getArrayCopy(), true);
    }
    public function get($index){
        return $this->offsetGet($index);
    }
    public function set($index, $object){
        $this->offsetSet($index, $object);
    }
    public function add($index, $object = null){
        if($object == null){
            $this->append($index);
        } else {
            $this->offsetSet($index, $object);
        }
    }
    public function addAll(ArrayObject $list){
        $arrays = $list->getArrayCopy();
        foreach($arrays as $value){
            $this->append($value);
        }
    }
    public function remove($index){
        $this->offsetUnset($index);
    }
    public function iterator(){
        return $this->getIterator();
    }
    public function toArray(){
        return $this->getArrayCopy();
    }
}

final class S2Dao_DatabaseMetaDataUtil {
    private function __construct() {
    }
    public static function getDbms(PDO $db){
        $dbms = S2Dao_DbmsManager::getDbms($db);
        if($dbms === null){
            throw new Exception('not such dbms case');
        }
        return $dbms;
    }
    public static function getDbMeta(PDO $pdo){
        return S2Dao_DBMetaDataFactory::create($pdo, self::getDbms($pdo));
    }
    public static function getPrimaryKeys(PDO $dbMetaData, $tableName) {
        return self::getPrimaryKeySet($dbMetaData, $tableName)->toArray();
    }
    public static function getPrimaryKeySet(PDO $dbMetaData, $tableName) {
        $schema = null;
        $index = strpos('.', $tableName);
        if ($index >= 0 && $index !== false) {
            $schema = substr($tableName, 0, $index);
            $tableName = substr($tableName, $index + 1);
        }
        $convertedTableName = self::convertIdentifier($dbMetaData, $tableName);
        $set = new S2Dao_ArrayList();
        self::addPrimaryKeys($dbMetaData,
                              self::convertIdentifier($dbMetaData, $schema),
                              $convertedTableName,
                              $set);
        if ($set->size() == 0) {
            self::addPrimaryKeys($dbMetaData, $schema, $tableName, $set);
        }
        if ($set->size() == 0 && $schema != null) {
            self::addPrimaryKeys($dbMetaData, null, $convertedTableName, $set);
            if ($set->size() == 0) {
                self::addPrimaryKeys($dbMetaData, null, $tableName, $set);
            }
        }
        return $set;
    }
    private static function addPrimaryKeys($dbMetaData, $schema, $tableName, $set) {
        try {
            $rs = self::getTableInfo($dbMetaData, $tableName, $schema);
            foreach($rs as $col){
                $col = array_change_key_case($col, CASE_LOWER);
                if(isset($col['flags']) &&
                        in_array(S2Dao_DBMetaData::PRIMARY_KEY, $col['flags'])){
                    $set->add($col['name']);
                }
            }
        } catch (Exception $ex) {
            throw new S2Dao_SQLRuntimeException($ex);
        }
    }
    public static function getColumns(PDO $dbMetaData, $tableName) {
        return self::getColumnSet($dbMetaData, $tableName)->toArray();
    }
    public static function getColumnMap(PDO $dbMetaData, $tableName) {
        $map = new S2Dao_HashMap();
        $set = self::getColumnSet($dbMetaData, $tableName);
        $c = $set->size();
        for($i = 0; $i < $c; ++$i){
            $column = $set->get($i);
            $lower = strtolower($column);
            $upper = strtoupper($column);
            $map->put($lower, null);
            $map->put($upper, null);
            $map->put($column, null);
        }
        return $map;
    }
    public static function getColumnSet(PDO $dbMetaData, $tableName) {
        $schema = null;
        $index = strpos('.', $tableName);
        if ($index >= 0 && $index !== false) {
            $schema = substr($tableName, 0, $index);
            $tableName = substr($tableName, $index + 1);
        }
        $convertedTableName = self::convertIdentifier($dbMetaData, $tableName);
        $set = new S2Dao_ArrayList();
        self::addColumns($dbMetaData,
                         self::convertIdentifier($dbMetaData, $schema),
                         $convertedTableName,
                         $set);
        if ($set->size() == 0) {
            self::addColumns($dbMetaData, $schema, $tableName, $set);
        }
        if ($set->size() == 0 && $schema != null) {
            self::addColumns($dbMetaData, null, $convertedTableName, $set);
            if ($set->size() == 0) {
                self::addColumns($dbMetaData, null, $tableName, $set);
            }
        }
        return $set;
    }
    private static function addColumns(PDO $pdo, $schema, $tableName, $set) {
        try {
            $rs = self::getTableInfo($pdo, $tableName, $schema);
            foreach($rs as $col){
                $col = array_change_key_case($col, CASE_LOWER);
                $set->add($col['name']);
            }
        } catch (Exception $ex) {
            throw new S2Dao_SQLRuntimeException($ex);
        }
    }
    public static function convertIdentifier(PDO $dbMetaData, $identifier) {
        $tables = S2Dao_ArrayUtil::spacetrim(self::getTables($dbMetaData));
        if (!in_array($identifier, $tables, true)) {
            $upper = strtoupper($identifier);
            $lower = strtolower($identifier);
            if (in_array($upper, $tables, true)) {
                return $upper;
            } else if(in_array($lower, $tables, true)){
                return $lower;
            } else {
                return $identifier;
            }
        }
        return $identifier;
    }
    private static function getTables(PDO $pdo){
        $stmt = $pdo->query(self::getDbms($pdo)->getTableSql());
        return $stmt->fetchAll(PDO::FETCH_COLUMN);
    }
    private static function getTableInfo(PDO $pdo, $table, $schema){
        return self::getDbMeta($pdo)->getTableInfo($table);
    }
}

class S2Dao_HashMap {
    private $element = array();
    public function size(){
        return count($this->element);
    }
    public function isEmpty(){
        return empty($this->element);
    }
    public function get($object){
        if(isset($this->element[$object])){
            return $this->element[$object];
        }
        return null;
    }
    public function put($key, $value){
        $this->element[$key] = $value;
    }
    public function remove($key){
        if(!$this->containsKey($key)){
            return null;
        }
        $element = $this->element[$key];
        unset($this->element[$key]);
        return $element;
    }
    public function containsKey($key){
        return array_key_exists($key, $this->element);
    }
    public function toArray(){
        return $this->element;
    }
    public function keySet(){
        $set = new S2Dao_ArrayList();
        $list = new ArrayObject(array_keys($this->toArray()));
        $set->addAll($list);
        return $set;
    }
}

?>
